﻿(function($) {
    xformSearch = function() {

        // object returned by function. Exposes public methods and variables.
        var pub = {};

        pub.getString = function(key) {
            return epi.shell.resource.get('EPiServer.Samples.XFormSearch.Strings', key);
        }

        pub.getSearchForm = function(e) {

            // Find the nearest parent form
            var gadgetInstance = e.data;
            var containerForm = $(this).closest("form");

            // Fetch new data and replace the existing table rows
            var formData = containerForm.serialize();

            // Add the gadgetId which lives outside of the form
            formData = formData + "&gadgetId=" + $("#gadgetId").val();

            gadgetInstance.setFeedbackMessage(xformSearch.getString('loadingFormDefinition'), { ajaxLoader: true });
            gadgetInstance.clearErrorMessage("");

            $.get(containerForm.attr("action"), formData, function(data, status) {
                gadgetInstance.clearFeedbackMessage();
                $("#SearchFormTable").html(data);
                $("#SearchResultsTable").html("");

                // Attach the date picker control to all the elements with 'datetime' class
                $(".datetime", "#SearchFormTable").datepicker({ changeMonth: true, changeYear: true });

                // Need to monitor the change event of all of the datetime search criteria drop downs
                $(".datetimesearchcriteria", "#SearchFormTable").bind("change", gadgetInstance, xformSearch.displayDateTimeControls);
            });
        };

        pub.displayDateTimeControls = function(e) {
            var controlName = $(this).attr("name");
            var searchRequirement = $(this).val();

            // Find the date time control set the calling control belongs to
            $(".datetimeRangeControls", "#SearchFormTable").each(function() {
                var controlIds = $(this).val().split('|');
                if (controlName == controlIds[0]) {
                    var input2Control = $("[name='" + controlIds[2] + "']");
                    switch (searchRequirement) {
                        case 'D><':
                        case 'D<>':
                            input2Control.closest("span").show();
                            break;

                        default:
                            input2Control.closest("span").hide();
                    }

                    return;
                }
            });
        };

        pub.searchData = function(control, page) {

            if (!xformSearch.validateSearchInput())
                return;

            var containerForm = $(control).closest("form");
            var gadgetInstance = epi.gadget.getByElement(containerForm);

            gadgetInstance.setFeedbackMessage(xformSearch.getString('searchingFormPosts'), { ajaxLoader: true });
            gadgetInstance.clearErrorMessage("");

            $("#currentPage", containerForm).val(page);
            var formData = containerForm.serialize();

            // Add the gadgetId which lives outside of the form
            formData = formData + "&gadgetId=" + $("#gadgetId").val();
            var outputType = $("input:checked[name='resultOption']", containerForm).val();

            if (outputType == 'CreateCsv' || outputType == 'CreateXml') {
                gadgetInstance.clearFeedbackMessage();
                window.location = containerForm.attr("action") + "?" + formData;
            }
            else {
                $.get(containerForm.attr("action"), formData, function(data, status) {
                    gadgetInstance.clearFeedbackMessage();
                    $("#SearchResultsTable", containerForm).html(data);
                    xformSearch.fixResultsPagingButtons();
                });
            }
        };

        pub.showResultsFirstPage = function(button) {
            xformSearch.searchData(button, "1");
        };

        pub.showResultsNextPage = function(button) {
            var page = $("#currentPage").val();
            xformSearch.searchData(button, ++page);
        };

        pub.showResultsPreviousPage = function(button) {
            var page = $("#currentPage").val();
            xformSearch.searchData(button, --page);
        };

        pub.showResultsLastPage = function(button) {
            var pages = $("#pageCount").val();
            xformSearch.searchData(button, pages);
        };

        pub.showResultsPage = function(button) {
            var page = parseInt($("input[name=goToPage]").val());
            if (isNaN(page) || page < 1 || page > parseInt($("#pageCount").val())) {
                $("#invalidPageNumber").show();
                return;
            }

            xformSearch.searchData(button, page);
        };

        pub.fixResultsPagingButtons = function() {
            var currentPage = parseInt($("#currentPage").val());
            var pages = parseInt($("#pageCount").val());

            if (currentPage == 1) {
                $(".xformSearchFirstPage").find("a").removeAttr("onclick");
                $(".xformSearchPreviousPage").find("a").removeAttr("onclick");
            }

            if (currentPage == pages) {
                $(".xformSearchNextPage").find("a").removeAttr("onclick");
                $(".xformSearchLastPage").find("a").removeAttr("onclick");
            }

            if (pages == 1) {
                $(".xformSearchGoToPage").find("a").removeAttr("onclick");
            }
        };

        pub.validateSearchInput = function() {
            $("#ValueRequiredLabel").hide();
            var valid = true;
            $(".datetimeRangeControls", "#SearchFormTable").each(function() {
                var controlIds = $(this).val().split('|');
                if (!xformSearch.validateDateRange($("[name='" + controlIds[0] + "']"), $("[name='" + controlIds[1] + "']"), $("[name='" + controlIds[2] + "']")))
                    valid = false;
            });
            return valid;
        };

        pub.validateDateRange = function(searchRequirementControl, date1Control, date2Control) {

            date1Control.attr("style", "border-color:inherit");
            date2Control.attr("style", "border-color:inherit");

            if (searchRequirementControl.val() == '') {
                return true;
            }

            var valid = true;

            if (date1Control.val() == '') {
                // adding a css class doesn't work because of the date picker functionality
                date1Control.attr("style", "border-color:Red");
                valid = false;
            }

            switch (searchRequirementControl.val()) {
                case 'D><':
                case 'D<>':
                    if (date2Control.val() == '') {
                        // adding a css class doesn't work because of the date picker functionality
                        date2Control.attr("style", "border-color:Red");
                        valid = false;
                    }
                    break;
            }

            if (!valid) {
                $("#ValueRequiredLabel").show();
            }

            return valid;
        };

        pub.init = function(e, gadgetInstance) {
            // Init method is called when this gadget instance is created.

            // Listen to the view loaded event to bind other events
            $(this).bind("epigadgetloaded", function(e, gadgetInstance) {
                if (gadgetInstance.routeParams && gadgetInstance.routeParams.action == "Configure") {
                    // If we are on the configure view, we do not want to bind the controls
                    return;
                }
                $("select[name=formId]", this).bind("change", gadgetInstance, xformSearch.getSearchForm);
                $("input[name=SearchButton]", this).bind("click", function(e) {
                    xformSearch.showResultsFirstPage(this);
                });
            });

            // Attach the date picker control to all the elements with 'datetime' class
            $(".datetime", this).datepicker({ changeMonth: true, changeYear: true });

            // Need to monitor the change event of all of the datetime search criteria drop downs
            $(".datetimesearchcriteria", this).bind("change", gadgetInstance, xformSearch.displayDateTimeControls);

            $(document).ajaxError(function(e, xhr, settings, exception) {
                gadgetInstance.clearFeedbackMessage();
                gadgetInstance.setErrorMessage(xhr.statusText);
            });

        };

        // Return our public methods
        return pub;
    } ();

} (epiJQuery));