﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using EPiServer.Data.Dynamic;
using EPiServer.Shell.Composition;

namespace EPiServer.Samples.XFormSearch.Models
{
    public class Settings
    {
        public Guid Id { get; set; }
        public Guid GadgetId { get; set; }
        public int ResultsPerPage { get; set; }
        public string CsvDelimiter { get; set; }

        /// <summary>
        /// Loads the settings for a gadget instance or create new settings with default data if there are no settings.
        /// </summary>
        /// <param name="gadgetId">The gadget id.</param>
        /// <returns>A <see cref="Settings"/> with information about the feed to load.</returns>
        public static Settings Load(DynamicDataStoreFactory storeFactory, Guid gadgetId)
        {
            Settings settings = GetSettingsStorage(storeFactory).Find<Settings>("GadgetId", gadgetId).FirstOrDefault();

            if (settings == null)
            {
                settings = new Settings();

                // Set default values
                settings.GadgetId = gadgetId;
                settings.ResultsPerPage = 10;
                settings.CsvDelimiter = ",";
            }

            return settings;
        }

        public void Save(DynamicDataStoreFactory storeFactory)
        {
            GetSettingsStorage(storeFactory).Save(this);
        }

        private static DynamicDataStore GetSettingsStorage(DynamicDataStoreFactory storeFactory)
        {
            return storeFactory.GetOrCreateStore(typeof(Settings));
        }            
    }
}
