﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using EPiServer.Data.Dynamic;
using EPiServer.Data;
using EPiServer.XForms;
using System.Xml;
using System.Diagnostics;

namespace EPiServer.Samples.XFormSearch.Models
{
    public enum SearchState
    {
        NotSearching,
        CountOnly,
        CountAndResults,
        DownloadingFile
    }

    public class ViewModel : Settings
    {
        public IList<SelectListItem> AvailableForms = new List<SelectListItem>();
        public IEnumerable<InputControlDescriptor> SelectedFormFields = new List<InputControlDescriptor>();
        public IList<SelectListItem> ResultsSortByOptions = new List<SelectListItem>();
        public IList<SelectListItem> ResultsSortOrderOptions = new List<SelectListItem>();
        public Guid SelectedFormId;
        public SearchState CurrentState;
        public IEnumerable<PropertyBag> Results;
        public int ResultCount;
        public int ResultsPage = 1;

        public static ViewModel Create(DynamicDataStoreFactory storeFactory, Settings settings, Guid? formId)
        {
            ViewModel selector = new ViewModel();
            DynamicDataStore store = storeFactory.GetStore("XForms");

            if (store != null)
            {
                var forms = (from item in store.ItemsAsPropertyBag() select new { Name = (string)item["FormName"], Id = ((Identity)item["Id"]).ExternalId }).ToList();

                foreach (var form in forms)
                {
                    selector.AvailableForms.Add(new SelectListItem()
                    {
                        Text = HttpUtility.HtmlDecode(form.Name),
                        Value = form.Id.ToString(),
                        Selected = formId.HasValue && formId.Value == form.Id
                    });
                }

                if (!formId.HasValue)
                {
                    formId = forms[0].Id;
                }

                selector.SelectedFormFields = XFormHelper.GetFormFields(formId.Value);
                selector.SelectedFormId = formId.Value;

                foreach (InputControlDescriptor desc in selector.SelectedFormFields)
                {
                    selector.ResultsSortByOptions.Add(new SelectListItem()
                    {
                         Text = desc.Label,
                         Value = desc.Name,
                         Selected = (selector.ResultsSortByOptions.Count == 0) ? true : false
                    });
                }

                selector.ResultsSortOrderOptions.Add(new SelectListItem()
                {
                    Text = LanguageHelper.Translate("/xformsearchgadget/output/ascending"),
                    Value = "A",
                    Selected = true
                });

                selector.ResultsSortOrderOptions.Add(new SelectListItem()
                {
                    Text = LanguageHelper.Translate("/xformsearchgadget/output/descending"),
                    Value = "D"
                });
            }

            selector.ResultsPerPage = settings.ResultsPerPage;
            return selector;
        }       
    }
}
