﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Xml;
using EPiServer.XForms;
using System.Diagnostics;
using EPiServer.Core;

namespace EPiServer.Samples.XFormSearch.Models
{
    public static class XFormHelper
    {
        public static List<InputControlDescriptor> GetFormFields(Guid formId)
        {
            List<InputControlDescriptor> descriptors = new List<InputControlDescriptor>();
            XForm form = XForm.CreateInstance(formId);
            var fields = form.InstanceRoot.ChildNodes.OfType<XmlNode>();
            InputControlDescriptor descriptor = null;

            foreach (var field in fields)
            {
                XmlNode controlNode = form.Document.SelectSingleNode(string.Format("//*[@ref='{0}']", field.Name));
                Debug.Assert(controlNode != null);

                descriptor = new InputControlDescriptor();
                descriptor.Name = field.Name;

                XmlNamespaceManager nsm = new XmlNamespaceManager(form.Document.NameTable);
                nsm.AddNamespace("xforms", "http://www.w3.org/2002/xforms");

                // Extract the label from the child label node
                XmlNode labelNode = controlNode.SelectSingleNode("./xforms:label", nsm);

                if (labelNode != null)
                {
                    descriptor.Label = labelNode.InnerText;
                }
                else
                {
                    descriptor.Label = descriptor.Name;
                }

                switch (controlNode.Name)
                {
                    case "xforms:input":
                    case "xforms:textarea":
                        descriptor.InputType = InputType.FreeText;
                        break;

                    case "xforms:select":
                        descriptor.InputType = InputType.MultiSelection;
                        descriptor.OptionPairs = GetOptionPairs(controlNode, nsm);
                        break;

                    case "xforms:select1":
                        descriptor.InputType = InputType.SingleSelection;
                        descriptor.OptionPairs = GetOptionPairs(controlNode, nsm);
                        break;
                }

                descriptors.Add(descriptor);
            }

            // Add the special 'fixed' fields that each XForm post has

            // Date Posted
            descriptor = new InputControlDescriptor();
            descriptor.Name = "Meta_DatePosted";
            descriptor.Label = LanguageHelper.Translate("/xformsearchgadget/searchInput/dateposted");
            descriptor.InputType = InputType.DateRange;
            descriptors.Add(descriptor);            

            // User Name
            descriptor = new InputControlDescriptor();
            descriptor.Name = "Meta_UserName";
            descriptor.Label = LanguageHelper.Translate("/xformsearchgadget/searchInput/username");
            descriptor.InputType = InputType.FreeText;
            descriptors.Add(descriptor);

            return descriptors;
        }

        private static List<KeyValuePair<string, string>> GetOptionPairs(XmlNode controlNode, XmlNamespaceManager nsm)
        {
            List<KeyValuePair<string, string>> optionPairs = new List<KeyValuePair<string, string>>();

            XmlNodeList itemsNodes = controlNode.SelectNodes("./xforms:item", nsm);

            foreach (XmlNode itemNode in itemsNodes)
            {
                KeyValuePair<string, string> pair = new KeyValuePair<string, string>(
                    itemNode.SelectSingleNode("./xforms:label", nsm).InnerText,
                    itemNode.SelectSingleNode("./xforms:value", nsm).InnerText);
                optionPairs.Add(pair);
            }

            return optionPairs;
        }
    }
}
